@@ -9,78 +9,17 @@
   You may redistribute it and/or modify it under the same
   license terms as Ruby.
 
-  $Id: ruby.rb,v 1.7 2006/06/11 15:36:20 mutoh Exp $
+  $Id: ruby.rb,v 1.9 2007/01/13 18:34:08 mutoh Exp $
 =end
 
 require 'irb/ruby-lex.rb'
 require 'stringio'
 
 class RubyLexX < RubyLex  # :nodoc: all
-  
-  def initialize
-    super
-    @prompt = nil
-    @here_header = nil
-    @lex_state = nil
-  end
-  
-  def read_escape
-    case ch = getc
-    when "\n", "\r", "\f"
-    when "\\", "n", "t", "r", "f", "v", "a", "e", "b" #"
-      return "\\".concat(ch)
-    when /[0-7]/
-      ungetc ch
-      3.times do
-        case ch = getc
-        when /[0-7]/
-        when nil
-          break
-        else
-          ungetc
-          break
-        end
-      end
-      
-    when "x"
-      2.times do
-        case ch = getc
-        when /[0-9a-fA-F]/
-        when nil
-          break
-        else
-          ungetc
-          break
-        end
-      end
-      
-    when "M"
-      if (ch = getc) != '-'
-        ungetc
-      else
-        if (ch = getc) == "\\" #"
-          read_escape
-        end
-      end
-      
-    when "C", "c", "^"
-      if ch == "C" and (ch = getc) != "-"
-        ungetc
-      elsif (ch = getc) == "\\" #"
-        read_escape
-      end
-    when '#'
-      return ch
-    else
-      # other characters
-    end
-  end
-
   # Parser#parse resemlbes RubyLex#lex
   def parse
     until (  (tk = token).kind_of?(RubyToken::TkEND_OF_SCRIPT) && !@continue or tk.nil?  )
       s = get_readed
- 
       if RubyToken::TkSTRING === tk
         def tk.value
           @value 
@@ -115,7 +54,6 @@ class RubyLexX < RubyLex  # :nodoc: all
       
       yield tk
     end
-    
     return nil
   end
 
@@ -142,47 +80,54 @@ module GetText
       target = nil
       msgid = nil
       line_no = nil
-      
-      rl.parse do |tk|
-        case tk
-        when RubyToken::TkIDENTIFIER, RubyToken::TkCONSTANT
-          if ID.include?(tk.name)
-            target = :normal
-          elsif PLURAL_ID.include?(tk.name)
-            target = :plural
-          else
-            target = nil
-          end
-          line_no = tk.line_no.to_s
-        when RubyToken::TkSTRING
-          if target
-            if msgid
-              msgid += tk.value
-            else
-              msgid = tk.value 
-            end
-          end
-        when RubyToken::TkPLUS, RubyToken::TkNL
-          #do nothing
-        when RubyToken::TkCOMMA
-          if msgid and target == :plural
-            msgid += "\000"
-            target = :normal
-          end
-        else
-          if msgid
-            key_existed = targets.assoc(msgid.gsub(/\n/, '\n'))
-            if key_existed 
-              targets[targets.index(key_existed)] = key_existed <<
-                file_name + ":" + line_no
-            else
-              targets << [msgid.gsub(/\n/, '\n'), file_name + ":" + line_no]
-            end
-            msgid = nil
-            target = nil
-          end
-        end
-        targets
+      tk = nil
+      begin
+	rl.parse do |tk|
+	  case tk
+	  when RubyToken::TkIDENTIFIER, RubyToken::TkCONSTANT
+	    if ID.include?(tk.name)
+	      target = :normal
+	    elsif PLURAL_ID.include?(tk.name)
+	      target = :plural
+	    else
+	      target = nil
+	    end
+	    line_no = tk.line_no.to_s
+	  when RubyToken::TkSTRING
+	    if target
+	      if msgid
+		msgid += tk.value
+	      else
+		msgid = tk.value 
+	      end
+	    end
+	  when RubyToken::TkPLUS, RubyToken::TkNL
+	    #do nothing
+	  when RubyToken::TkCOMMA
+	    if msgid and target == :plural
+	      msgid += "\000"
+	      target = :normal
+	    end
+	  else
+	    if msgid
+	      key_existed = targets.assoc(msgid.gsub(/\n/, '\n'))
+	      if key_existed 
+		targets[targets.index(key_existed)] = key_existed <<
+		  file_name + ":" + line_no
+	      else
+		targets << [msgid.gsub(/\n/, '\n'), file_name + ":" + line_no]
+	      end
+	      msgid = nil
+	      target = nil
+	    end
+	  end
+	  targets
+	end
+      rescue
+	$stderr.print "\n\nError: #{$!.inspect} "
+        $stderr.print " in #{file_name}:#{tk.line_no}\n\t #{lines[tk.line_no - 1]}" if tk
+        $stderr.print "\n"
+	exit
       end
       targets
     end
