@@ -6,12 +6,11 @@
   You may redistribute it and/or modify it under the same
   license terms as Ruby.
 
-  $Id: rails.rb,v 1.43 2006/07/16 15:19:17 mutoh Exp $
+  $Id: rails.rb,v 1.55 2007/01/21 15:16:06 mutoh Exp $
 =end
 
 require 'gettext/cgi'
-require 'active_record'
-require 'active_support'
+require 'action_controller'
 
 module GetText
   # GetText::Rails supports Ruby on Rails.
@@ -53,76 +52,112 @@ module GetText
     #   * :charset - the charset. Generally UTF-8 is recommanded.
     #     And the charset is set order by "the argument of bindtextdomain" 
     #     > HTTP_ACCEPT_CHARSET > Default charset(UTF-8).
-    #   * :with_model - false if you want to ignore ActiveRecord support.
-    #   * :with_helper - false if you want to ignore ApplicationHelper support.
     #
     # Note: Don't use locale, charset, with_model argument(not in options). 
     # They are remained for backward compatibility.
     #
     def bindtextdomain(domainname, options = {}, locale = nil, charset = nil, with_model = true)
-      opt = {:with_helper => true, :with_model => true}
+      opt = {}
       if options.kind_of? CGI
 	# For backward compatibility
-	opt.merge!(:cgi => options, :locale => locale, :charset => charset, :with_model => with_model)
+	opt.merge!(:cgi => options, :locale => locale, :charset => charset)
       else
 	opt.merge!(options)
       end
       opt[:path] ||= File.join(RAILS_ROOT, "locale")
-      Locale.set_current(nil)  # IMPORTANT! current locale should be nil once(default is used).
-      set_cgi(opt[:cgi]) if opt[:cgi]
-      ret = _bindtextdomain(domainname, opt)
-      bindtextdomain_to(ActiveRecord::Base, domainname) if opt[:with_model]
-      bindtextdomain_to(ApplicationHelper, domainname) if opt[:with_helper]
-      ret
-    end
-
-    def bindtextdomain_to(klass, domainname) #:nodoc:
-      klass.class_eval {
-	include GetText
-	textdomain(domainname)
-      }
+      _bindtextdomain(domainname, opt)
     end
   end
 end
 
-
 module ActionController #:nodoc:
   class Base
     helper GetText::Rails
     include GetText::Rails
 
-    @@gettext_domainname = nil
+    @@gettext_domainnames = []
     @@gettext_content_type = nil
-
+    
     prepend_before_filter :init_gettext
     after_filter :init_content_type
-    
-    def init_gettext_main #:nodoc:
-      if defined? request.cgi
-	cgi_ = request.cgi
-	cgi_.params["lang"] = [@params["lang"]] if @params["lang"]
-	bindtextdomain(@@gettext_domainname, {:cgi => cgi_})  #You need to pass CGI object first.
-      else
-	bindtextdomain(@@gettext_domainname, {:locale => @params["lang"]})
-      end
+     
+    def init_gettext_main(cgi) #:nodoc:
+      cgi.params["lang"] = [params["lang"]] if params["lang"]
+      set_cgi(cgi)
+      set_locale_all(nil)
     end
 
     def init_content_type #:nodoc:
-      if @headers["Content-Type"] and /javascript/ =~ @headers["Content-Type"]
-	@headers["Content-Type"] = "text/javascript; charset=#{GetText.output_charset}"
-      elsif ! @headers["Content-Type"]
-	@headers["Content-Type"] = "#{@@gettext_content_type}; charset=#{GetText.output_charset}"
+      if headers["Content-Type"] and /javascript/ =~ headers["Content-Type"]
+	headers["Content-Type"] = "text/javascript; charset=#{GetText.output_charset}"
+      elsif ! headers["Content-Type"]
+	headers["Content-Type"] = "#{@@gettext_content_type}; charset=#{GetText.output_charset}"
       end
     end
 
+    def call_methods_around_init_gettext(ary)  #:nodoc:
+      ary.each do |block|
+	if block.kind_of? Symbol
+	  send(block)
+	else
+	  block.call(self)
+	end
+      end
+    end
 
     def init_gettext # :nodoc:
-      init_gettext_main if @@gettext_domainname
-      ActiveRecord::Errors.class_eval{
-	# You need to call bindtextdomain here because ActiveRecord::Errors doesn't know
-	# what locale is used in.
-	bindtextdomain("rails")
-      }
+      cgi = nil
+      if defined? request.cgi
+        cgi = request.cgi
+      end
+      call_methods_around_init_gettext(@@before_init_gettext)
+      init_gettext_main(cgi) if @@gettext_domainnames.size > 0
+      call_methods_around_init_gettext(@@after_init_gettext)
+
+      if ::RAILS_ENV == "development"
+	@@before_init_gettext = []
+	@@after_init_gettext = []
+      end
+    end
+
+    # Append a block which is called before initializing gettext on the each WWW request.
+    #
+    # (e.g.)
+    #   class ApplicationController < ActionController::Base
+    #     before_init_gettext{|controller|
+    #       cookies = controller.cookies
+    #       if (cookies["lang"].nil? or cookies["lang"].empty?)
+    #         GetText.locale = "zh_CN"
+    #       else
+    #         GetText.locale = cookies["lang"]
+    #       end
+    #     }
+    #     init_gettext "myapp"
+    #     # ...
+    #   end
+    @@before_init_gettext = []
+    def self.before_init_gettext(*methods, &block)
+      @@before_init_gettext += methods
+      @@before_init_gettext << block if block_given? 
+    end
+
+    # Append a block which is called after initializing gettext on the each WWW request.
+    #
+    # The GetText.locale is set the locale which bound to the textdomains
+    # when gettext is initialized.
+    #
+    # (e.g.)
+    #   class ApplicationController < ActionController::Base
+    #     after_init_gettext {|controller|
+    #       L10nClass.new(GetText.locale)
+    #     }
+    #     init_gettext "foo"
+    #     # ...
+    #   end
+    @@after_init_gettext = []
+    def self.after_init_gettext(*methods, &block)
+      @@after_init_gettext += methods
+      @@after_init_gettext << block if block_given? 
     end
     
     # Bind a 'textdomain' to all of the controllers/views/models. Call this instead of GetText.bindtextdomain.
@@ -130,8 +165,9 @@ module ActionController #:nodoc:
     # * options: options as a Hash.
     #   * :charset - the output charset. Default is "UTF-8"
     #   * :content_type - the content type. Default is "text/html"
+    #   * :locale_path - the path to locale directory. Default is {RAILS_ROOT}/locale or {plugin root directory}/locale.
     #
-    # locale is searched the order by @params["lang"] > "lang" value of QUERY_STRING > 
+    # locale is searched the order by params["lang"] > "lang" value of QUERY_STRING > 
     # "lang" value of Cookie > HTTP_ACCEPT_LANGUAGE value > Default locale(en). 
     # And the charset is set order by "the argument of bindtextdomain" > HTTP_ACCEPT_CHARSET > Default charset(UTF-8).
     #
@@ -153,243 +189,66 @@ module ActionController #:nodoc:
       if options.kind_of? String
 	# For backward compatibility
 	opt.merge!(:charset => options, :content_type => content_type)
+      else
+	opt.merge!(options)
       end
       GetText.output_charset = opt[:charset]
-      @@gettext_domainname = domainname
       @@gettext_content_type = opt[:content_type]
+      locale_path = opt[:locale_path]
+      unless locale_path
+	cal = caller[0]
+	if cal =~ /app.controllers/
+	  locale_path = File.join(cal.split(/app.controllers/)[0] + "locale")
+	else
+	  locale_path = File.join(RAILS_ROOT, "locale")
+	end
+      end
+
+      unless @@gettext_domainnames.find{|i| i[0] == domainname}
+	@@gettext_domainnames << [domainname, locale_path] 
+      end
+
+      bindtextdomain(domainname, {:path => locale_path})
+      if defined? ActiveRecord::Base
+	textdomain_to(ActiveRecord::Base, domainname) 
+	textdomain_to(ActiveRecord::Validations, domainname)
+      end
+      textdomain_to(ActionView::Base, domainname) if defined? ActionView::Base
+      textdomain_to(ApplicationHelper, domainname) if defined? ApplicationHelper
+      textdomain_to(ActionMailer::Base, domainname) if defined? ActionMailer::Base
     end
 
     # Gets the textdomain name of this controller.
+    # This returns the first textdomain which is bound in app/controller/*.rb.
+    #
+    # *Notice* Deprecated since 1.8.
     def self.textdomainname
-      @@gettext_domainname
+      textdomain = @@gettext_domainnames[0]
+      textdomain ? textdomain[0] : nil
     end
 
+    # Gets the textdomain name and path of this controller which is set 
+    # with init_gettext. *(Since 1.8)*
+    # 
+    # * Returns: [[textdomainname1, path1], [textdomainname2, path2], ...]
+    def self.textdomains
+      @@gettext_domainnames
+    end
   end
 
   class TestRequest < AbstractRequest  #:nodoc:
     @cgi = nil
     def cgi
-      @cgi = CGI.new unless @cgi
+      unless @cgi
+        ENV['REQUEST_URI'] ||= "http://localhost:3000/"
+        @cgi = CGI.new 
+      end
       @cgi
     end
   end
+
 end
 
-module ActiveRecord #:nodoc:
-  class Migration
-    extend GetText
-    include GetText
-  end
-  
-  module ConnectionAdapters #:nodoc:
-    # An abstract definition of a column in a table.
-    class Column
-      attr_accessor :table_class
-
-      def human_name
-        table_class.human_attribute_name(@name)
-      end
-    end
-  end
-
-  module Validations # :nodoc:
-    def self.append_features(base) # :nodoc:
-      super
-      base.extend ClassMethods
-      base.class_eval{
-	def gettext(str)  #:nodoc:
-	  _(str)
-	end
-	def self.human_attribute_name(attribute_key_name) #:nodoc:
-	  s_("#{self}|#{attribute_key_name.humanize}")
-	end
-
-	def self.human_attribute_table_name_for_error(table_name) #:nodoc:
-	  _(table_name.gsub(/_/, " "))
-	end
-      }
-    end
-
-    module ClassMethods #:nodoc:
-      @@custom_error_messages_d = {}
-      
-      # Very ugly but...
-      def validates_length_of(*attrs)  #:nodoc:
-	if attrs.last.is_a?(Hash)
-	  msg = attrs.last[:message] 
-	  if msg
-	    @@custom_error_messages_d[msg] = /#{Regexp.escape(msg).sub(/%d/, '(\d+)')}/ 
-	  end
-	end
-	validates_size_of(*attrs)
-      end
-      
-      def custom_error_messages_d  #:nodoc:
-	@@custom_error_messages_d
-      end
-    end
-    def custom_error_messages_d  #:nodoc:
-      self.class.custom_error_messages_d
-    end
-  end
-  
-  class Base
-    include GetText::Rails
-    include Validations
-
-    @@gettext_untranslate = false
-    @@gettext_untranslate_columns = []
-
-    # Untranslate all of the tablename/fieldnames in this model class.
-    def self.untranslate_all
-      @@gettext_untranslate = true
-    end
-
-    # Returns true if "untranslate_all" is called. Otherwise false.
-    def self.untranslate_all?
-      @@gettext_untranslate
-    end
-
-    # Sets the untranslate columns.
-    # (e.g.) untranslate :foo, :bar, :baz
-    def self.untranslate(*w)
-      @@gettext_untranslate_columns += w.collect{|v| v.to_s}
-    end
-
-    # Returns true if the column is set "untranslate".
-    # (e.g.) untranslate? :foo
-    def self.untranslate?(columnname)
-      @@gettext_untranslate_columns.include?(columnname)
-    end
-
-    def self.untranslate_data #:nodoc:
-      [@@gettext_untranslate, @@gettext_untranslate_columns]
-    end
-
-    def self.columns
-      unless @columns
-	@columns = connection.columns(table_name, "#{name} Columns")
-	@columns.each {|column| 
-	  column.table_class = self
-	  column.primary = column.name == primary_key
-	}
-      end
-      @columns
-    end
-    
-    # call-seq:
-    # set_error_message_title(msg)
-    #
-    # ((*Deprecated*)) 
-    # Use ActionView::Helpers::ActiveRecordHelper::L10n.set_error_message_title
-    # instead.
-    #
-    # Sets a your own title of error message dialog.
-    # * msg: [single_msg, plural_msg]. Usually you need to call this with Nn_().
-    # * Returns: [single_msg, plural_msg]
-    def self.set_error_message_title(msg, plural_msg = nil)
-      ActionView::Helpers::ActiveRecordHelper::L10n.set_error_message_title(msg, plural_msg)
-    end
-
-    # call-seq:
-    # set_error_message_explanation(msg)
-    #
-    # ((*Deprecated*)) 
-    # Use ActionView::Helpers::ActiveRecordHelper::L10n.set_error_message_explanation
-    # instead.
-    #
-    # Sets a your own explanation of the error message dialog.
-    # * msg: [single_msg, plural_msg]. Usually you need to call this with Nn_().
-    # * Returns: [single_msg, plural_msg]
-    def self.set_error_message_explanation(msg, plural_msg = nil)
-      ActionView::Helpers::ActiveRecordHelper::L10n.set_error_message_explanation(msg, plural_msg)
-    end
-  end
-  
-  # activerecord-1.14.3/lib/active_record/validations.rb
-  class Errors #:nodoc:
-    include GetText
-
-    RE_FIELD_NAME = /%\{fn\}/  #:nodoc:
-    
-    # You need to define this here, because this values will be updated by application.
-    default_error_messages.update(
-				  :inclusion => N_("%{fn} is not included in the list"),
-				  :exclusion => N_("%{fn} is reserved"),
-				  :invalid => N_("%{fn} is invalid"),
-				  :confirmation => N_("%{fn} doesn't match confirmation"),
-				  :accepted  => N_("%{fn} must be accepted"),
-				  :empty => N_("%{fn} can't be empty"),
-				  :blank => N_("%{fn} can't be blank"),
-				  :too_long => N_("%{fn} is too long (max is %d characters)"),  
-				  :too_short => N_("%{fn} is too short (min is %d characters)"), 
-				  :wrong_length => N_("%{fn} is the wrong length (should be %d characters)"),
-				  :taken => N_("%{fn} has already been taken"),
-				  :not_a_number => N_("%{fn} is not a number")
-				  )
-    @@default_error_messages_d = {
-      default_error_messages[:too_long] => /#{Regexp.escape(default_error_messages[:too_long]).sub(/%d/, '(\d+)')}/,
-      default_error_messages[:too_short] =>/#{Regexp.escape(default_error_messages[:too_short]).sub(/%d/, '(\d+)')}/,
-      default_error_messages[:wrong_length] =>/#{Regexp.escape(default_error_messages[:wrong_length]).sub(/%d/, '(\d+)')}/,
-    }
-    cattr_accessor :default_error_messages_d
-
-    def localize_error_messages # :nodoc:
-      errors = {}
-      @errors.each_key do |attr|
-        @errors[attr].each do |msg|
-          next if msg.nil?
-          custom_msg = nil
-          #Ugly but... :-<
-          @@default_error_messages_d.merge(@base.custom_error_messages_d).each do |key, regexp|
-            if regexp =~ msg
-              custom_msg = @base.gettext(key)
-              custom_msg = _(msg) if custom_msg == msg 
-	      custom_msg = _(custom_msg) % $1.to_i
-	      break
-            end
-          end
-          unless custom_msg
-            custom_msg = @base.gettext(msg)
-            custom_msg = _(msg) if custom_msg == msg 
-          end
-          if attr == "base"
-            full_message = custom_msg
-          elsif  RE_FIELD_NAME =~ custom_msg
-            full_message = custom_msg % {:fn => @base.class.human_attribute_name(attr)}
-          else
-            full_message = @base.class.human_attribute_name(attr) + " " + custom_msg
-          end
-	  errors[attr] = full_message
-        end
-      end
-      errors
-    end
-
-    def on(attribute) #:nodoc:
-      errors = localize_error_messages
-      if errors[attribute.to_s].nil?
-        nil
-      elsif errors[attribute.to_s].length == 1
-        errors[attribute.to_s].first
-      else
-        errors[attribute.to_s]
-      end
-    end
-
-    def full_messages #:nodoc:
-      full_messages = []
-      errors = localize_error_messages
-      errors.each do |attr, msg|
-	next if msg.nil?
-	full_messages << msg
-      end
-      full_messages
-    end
-  end
-end
-
-
 module ActionView #:nodoc:
   class Base #:nodoc:
     alias render_file_without_locale render_file #:nodoc:
@@ -502,8 +361,6 @@ module ActionView #:nodoc:
       include GetText
       alias distance_of_time_in_words_without_locale distance_of_time_in_words #:nodoc:
 
-      bindtextdomain("rails")
-
       # This is FAKE constant. The messages are found by rgettext as the msgid. 
       MESSAGESS = [N_('less than 5 seconds'), N_('less than 10 seconds'), N_('less than 20 seconds'),
 	N_('half a minute'), N_('less than a minute')]
@@ -512,6 +369,8 @@ module ActionView #:nodoc:
       DAYS    = [/^(\d+) days?$/, Nn_('1 day', '%{num} days')]
 
       def distance_of_time_in_words(from_time, to_time = 0, include_seconds = false)
+	textdomain("rails")
+	
 	msg = distance_of_time_in_words_without_locale(from_time, to_time, include_seconds)
 	match = false
 	[MINUTES, HOURS, DAYS].each do |regexp, nn|
@@ -534,7 +393,7 @@ if defined? ActionMailer
       include GetText::Rails 
       extend GetText::Rails
       
-      alias :_create! :create! #:nodoc:
+      alias :create_without_gettext! :create! #:nodoc:
       
       def base64(text, charset="iso-2022-jp", convert=true)
 	if convert
@@ -547,7 +406,7 @@ if defined? ActionMailer
       end
       
       def create!(*arg) #:nodoc:
-	_create!(*arg)
+	create_without_gettext!(*arg)
 	if Locale.get.language == "ja"
 	  require 'nkf'
 	  @mail.subject = base64(@mail.subject)
@@ -575,3 +434,5 @@ end
 if ::RAILS_ENV == "development"
   GetText::TextDomain.check_mo = true
 end
+
+require 'gettext/active_record'
